#ifndef FrameCPP_VERSION_8_FrAdcData_HH
#define FrameCPP_VERSION_8_FrAdcData_HH

#if defined(__cplusplus)
#if ! defined(SWIG)
#include <memory>
#include <vector>

#include "general/types.hh"

#include "framecpp/Common/Container.hh"
#include "framecpp/Common/TOCInfo.hh"
#endif /* ! defined(SWIG) */
#endif /* defined(__cplusplus) */

#include "framecpp/Version7/FrAdcData.hh"

#if defined(__cplusplus)
#if ! defined(SWIG)
#include "framecpp/Version8/FrameSpec.hh"
#include "framecpp/Version8/Dimension.hh"
#include "framecpp/Version8/FrVect.hh"
#include "framecpp/Version8/STRING.hh"
#endif /* ! defined(SWIG) */
#endif /* defined(__cplusplus) */

#if defined(__cplusplus) && !defined(SWIGIMPORTED)
namespace FrameCPP
{
  namespace Version_8
  {
    const FR_ADC_DATA_BIAS_TYPE FR_ADC_DATA_DEFAULT_BIAS = 0.0;
    const FR_ADC_DATA_DATAVALID_TYPE FR_ADC_DATA_DEFAULT_DATA_VALID = 0;
    const FR_ADC_DATA_FSHIFT_TYPE FR_ADC_DATA_DEFAULT_FSHIFT = 0.0;
    const FR_ADC_DATA_PHASE_TYPE FR_ADC_DATA_DEFAULT_PHASE = 0.0;
    const FR_ADC_DATA_SLOPE_TYPE FR_ADC_DATA_DEFAULT_SLOPE = 1.0;
    const FR_ADC_DATA_TIMEOFFSET_TYPE FR_ADC_DATA_DEFAULT_TIME_OFFSET = 0.0;
    inline const char* FR_ADC_DATA_DEFAULT_UNITS( )
    {
      return "counts";
    }

    //===================================================================
    /// \brief ADC Data that does not reference a pointer
    //===================================================================
    class FrAdcDataNonPtr
    {
    public:
      typedef STRING				name_type;
      typedef STRING				comment_type;
      typedef FR_ADC_DATA_CHANNEL_GROUP_TYPE	channelGroup_type;
      typedef FR_ADC_DATA_CHANNEL_NUMBER_TYPE	channelNumber_type;
      typedef FR_ADC_DATA_N_BITS_TYPE		nBits_type;
      typedef FR_ADC_DATA_BIAS_TYPE		bias_type;
      typedef FR_ADC_DATA_SLOPE_TYPE		slope_type;
      typedef STRING				units_type;
      typedef FR_ADC_DATA_SAMPLERATE_TYPE	sampleRate_type;
      typedef FR_ADC_DATA_TIMEOFFSET_TYPE	timeOffset_type;
      typedef FR_ADC_DATA_FSHIFT_TYPE		fShift_type;
      typedef FR_ADC_DATA_PHASE_TYPE		phase_type;
      typedef FR_ADC_DATA_DATAVALID_TYPE	dataValid_type;


      static const bias_type		DEFAULT_BIAS;
      static const slope_type		DEFAULT_SLOPE;
      static const fShift_type		DEFAULT_FSHIFT;
      static const timeOffset_type	DEFAULT_TIME_OFFSET;
      static const dataValid_type	DEFAULT_DATA_VALID;
      static const phase_type		DEFAULT_PHASE;

      //-----------------------------------------------------------------
      /// \brief Extend the comment associated with the FrAdcData structure.
      //-----------------------------------------------------------------
      void AppendComment( const comment_type& Comment );

      //-----------------------------------------------------------------
      /// \brief Return the name associate with the FrAdcData structure.
      ///
      /// \return
      ///     The name associated with the FrAdcData structure
      //-----------------------------------------------------------------
      const name_type& GetName( ) const; 

      //-----------------------------------------------------------------
      /// \brief A brief description of the FrAdcData structure
      ///
      /// \return
      ///     A brief description of the FrAdcData structure
      //-----------------------------------------------------------------
      const comment_type& GetComment( ) const;

      //-----------------------------------------------------------------
      /// \brief The channel number for the FrAdcData
      ///
      /// \return
      ///     The channel number for the FrAdcData
      //-----------------------------------------------------------------
      channelNumber_type GetChannelNumber( ) const;

      //-----------------------------------------------------------------
      /// \brief The channel group for the FrAdcData
      ///
      /// \return
      ///     The channel group for the FrAdcData
      //-----------------------------------------------------------------
      channelGroup_type GetChannelGroup( ) const;

      //-----------------------------------------------------------------
      /// \brief Tne number of bits in the A/D output.
      ///
      /// \return
      ///      Tne number of bits in the A/D output.
      //-----------------------------------------------------------------
      nBits_type GetNBits( ) const;

      //-----------------------------------------------------------------
      /// \brief DC bias on channel.
      ///
      /// \return
      ///     DC bias on channel.
      //-----------------------------------------------------------------
      bias_type GetBias( ) const;

      //-----------------------------------------------------------------
      /// \brief ADC calibration units per count.
      ///
      /// \return
      ///     ADC calibration units per count.
      //-----------------------------------------------------------------
      slope_type GetSlope( ) const;

      //-----------------------------------------------------------------
      /// \brief The units used in the ADC calibration
      ///
      /// \return
      ///     The units used in the ADC calibration
      //-----------------------------------------------------------------
      STRING GetUnits( ) const;

      //-----------------------------------------------------------------
      /// \brief The data acquisition rate.
      ///
      /// \return
      ///     The data acquisition rate.
      //-----------------------------------------------------------------
      sampleRate_type GetSampleRate( ) const;

      //-----------------------------------------------------------------
      /// \brief The offset of the first sample relative to the frame start time.
      ///
      /// \return
      ///     The offset of the first sample relative to the frame
      ///     start time.
      //-----------------------------------------------------------------
      timeOffset_type GetTimeOffset( ) const;

      //-----------------------------------------------------------------
      /// \brief The frequency in the original data corresponding to 0 Hz.
      ///
      /// \return
      ///     The frequency in the original data corresponding to 0 Hz.
      //-----------------------------------------------------------------
      fShift_type GetFShift( ) const;
      
      //-----------------------------------------------------------------
      /// \brief The phase of hetrodyning signal at start of the dataset.
      ///
      /// \return
      ///     The phase of hetrodyning signal at start of the dataset.
      //-----------------------------------------------------------------
      phase_type GetPhase( ) const;

      //-----------------------------------------------------------------
      /// \brief Retrieve the dataValid value.
      ///
      /// \return
      ///     The value of the dataValid field.
      //-----------------------------------------------------------------
      dataValid_type GetDataValid( ) const;

      //-----------------------------------------------------------------
      /// \brief Set the numeric group id for the channel
      //-----------------------------------------------------------------
      void SetChannelGroup( channelGroup_type Value);

      //-----------------------------------------------------------------
      /// \brief Set the numeric id for the channel
      //-----------------------------------------------------------------
      void SetChannelNumber( channelNumber_type Value);

      //-----------------------------------------------------------------
      /// \brief Set the comment associated with the FrAdcData structure.
      //-----------------------------------------------------------------
      void SetComment( const comment_type& comment );

      //-----------------------------------------------------------------
      /// \brief Establish the value of the dataValid field.
      ///
      /// \param[in] flag
      ///     The desired value of the dataValid field.
      //-----------------------------------------------------------------
      void SetDataValid( dataValid_type flag );

      //-----------------------------------------------------------------
      /// \brief Establish the value of the bias field.
      ///
      /// \param[in] bias
      ///     The desired value of the bias field.
      //-----------------------------------------------------------------
      void SetBias( bias_type bias );

      //-----------------------------------------------------------------
      /// \brief The number of bits in the A/D output.
      ///
      /// \param[in] NBits
      ///      Tne number of bits in the A/D output.
      //-----------------------------------------------------------------
      void SetNBits( nBits_type NBits );

      //-----------------------------------------------------------------
      /// \brief Establish the value of the sampleRate field.
      ///
      /// \param[in] SampleRate
      ///     The desired value of the sampleRate field.
      //-----------------------------------------------------------------
      void SetSampleRate( sampleRate_type SampleRate );

      //-----------------------------------------------------------------
      /// \brief Establish the value of the slope field.
      ///
      /// \param[in] slope
      ///     The desired value of the slope field.
      //-----------------------------------------------------------------
      void SetSlope( slope_type slope );

      //-----------------------------------------------------------------
      /// \brief Establish the value of the timeOffset field.
      ///
      /// \param[in] time
      ///     The desired value of the timeOffset field.
      //-----------------------------------------------------------------
      void SetTimeOffset( timeOffset_type time );

    protected:
      //---------------------------------------------------------------
      /// Channel name  -- must be unique with the frame
      //---------------------------------------------------------------
      name_type	name;
      //---------------------------------------------------------------
      /// Comment for the ADC.
      //---------------------------------------------------------------
      comment_type	comment;
      //---------------------------------------------------------------
      /// Channel grouping number containing ADC.
      ///
      /// \note
      ///     This variable is determined by the site and must be
      ///     unique over all detectors.
      //---------------------------------------------------------------
      channelGroup_type	channelGroup;
      //---------------------------------------------------------------
      /// Channel number
      ///
      /// \note
      ///     This variable is determined by the site and must be
      ///     unique over all detectors.
      //---------------------------------------------------------------
      channelNumber_type	channelNumber;
      //---------------------------------------------------------------
      /// Number of bits in A/D output.
      //---------------------------------------------------------------
      nBits_type	nBits;
      //---------------------------------------------------------------
      /// DC bias on channel (Units @ ADC counts = 0)
      //---------------------------------------------------------------
      bias_type	bias;
      //---------------------------------------------------------------
      /// ADC calibration: input units/count.
      //---------------------------------------------------------------
      slope_type	slope;
      //---------------------------------------------------------------
      /// ADC calibration: input units for slope
      /// If dimensionless, then units == \<NONE\>, in CAPITALS
      /// (without \<...\>).
      //---------------------------------------------------------------
      STRING	units;
      //---------------------------------------------------------------
      /// Data acquisition rate, samples per second.
      //---------------------------------------------------------------
      sampleRate_type	sampleRate;
      //---------------------------------------------------------------
      /// Offset of first sample relative to the frame start time
      /// in seconds.
      /// The value must be positive and smaller than the frame length.
      ///
      /// \note
      ///     Time offsets always added together (to obtain decimal
      ///     time offset: time of the first sample equals
      ///     GTime + timeOffset.
      ///     All quantities are ALWAYS positive.
      //---------------------------------------------------------------
      timeOffset_type	timeOffset;
      //---------------------------------------------------------------
      /// The frequency (in HZ) in the original data that corresponds
      /// to zero Hz in the heterodyned series.
      ///
      /// \note
      ///     In the heterodyning process the real time series in
      ///     multiplied by cos[2pi fShift(t-to) + phase] to get
      ///     the real part and by -sin[2pi fShift(t-t(0)) + phase] to
      ///     get the imaginary part of the resulting complex time
      ///     series.
      ///     The time origin t(0) is the beginning of the frame.
      //---------------------------------------------------------------
      fShift_type	fShift;
      //---------------------------------------------------------------
      /// Phase (in radian) of heterodyning signal at start of dataset.
      ///
      /// \note
      ///     In the heterodyning process the real time series in
      ///     multiplied by cos[2pi fShift(t-to) + phase] to get
      ///     the real part and by -sin[2pi fShift(t-t(0)) + phase] to
      ///     get the imaginary part of the resulting complex time
      ///     series.
      ///     The time origin t(0) is the beginning of the frame.
      //---------------------------------------------------------------
      phase_type	phase;
      //---------------------------------------------------------------
      /// Data valid flag: If the value is zero, the ADC data is valid.
      /// If the value is non-zero, the ADC data should be suspect
      /// or not valid.
      //---------------------------------------------------------------
      dataValid_type	dataValid;

      //-----------------------------------------------------------------
      /// \brief Default constructor
      //-----------------------------------------------------------------
      FrAdcDataNonPtr( );

      //-----------------------------------------------------------------
      /// \brief Initialzie the instance from an input stream
      //-----------------------------------------------------------------
      FrAdcDataNonPtr( Common::IStream& Stream );

      //-----------------------------------------------------------------
      /// \brief Number of bytes needed to write this structure
      ///
      /// \param[in] Stream
      ///     The stream from which to the object is being read or
      ///     written.
      ///
      /// \return
      ///     The number of bytes need to read or write this object.
      //-----------------------------------------------------------------
      cmn_streamsize_type bytes( const Common::StreamBase& Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Write contents to the stream
      //-----------------------------------------------------------------
      void write( Common::OStream& Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Comparison operator
      //-----------------------------------------------------------------
      bool operator==( const FrAdcDataNonPtr& RHS ) const;
    };

    inline const FrAdcDataNonPtr::name_type& FrAdcDataNonPtr::
    GetName( ) const
    {
      return name;
    }

    inline const FrAdcDataNonPtr::comment_type& FrAdcDataNonPtr::
    GetComment( ) const
    {
      return comment;
    }

    inline FrAdcDataNonPtr::channelNumber_type FrAdcDataNonPtr::
    GetChannelNumber( ) const
    { 
      return channelNumber;
    }

    inline FrAdcDataNonPtr::channelGroup_type FrAdcDataNonPtr::
    GetChannelGroup( ) const
    {
      return channelGroup;
    }

    inline FrAdcDataNonPtr::nBits_type FrAdcDataNonPtr::
    GetNBits( ) const
    {
      return nBits;
    }

    inline FrAdcDataNonPtr::bias_type FrAdcDataNonPtr::
    GetBias( ) const
    {
      return bias;
    }

    inline FrAdcDataNonPtr::slope_type FrAdcDataNonPtr::
    GetSlope( ) const 
    {
      return slope;
    }

    inline STRING FrAdcDataNonPtr::
    GetUnits( ) const
    {
      return units;
    }

    inline FrAdcDataNonPtr::sampleRate_type FrAdcDataNonPtr::
    GetSampleRate( ) const
    {
      return sampleRate;
    }

    inline FrAdcDataNonPtr::timeOffset_type FrAdcDataNonPtr::
    GetTimeOffset( ) const
    {
      return timeOffset;
    }

    inline FrAdcDataNonPtr::fShift_type FrAdcDataNonPtr::
    GetFShift( ) const
    {
      return fShift;
    }

    inline FrAdcDataNonPtr::phase_type FrAdcDataNonPtr::
    GetPhase( ) const
    {
      return phase;
    }

    inline FrAdcDataNonPtr::dataValid_type FrAdcDataNonPtr::
    GetDataValid( ) const
    {
      return dataValid;
    }

    inline void FrAdcDataNonPtr::
    SetDataValid( dataValid_type DataValid )
    {
      dataValid = DataValid;
    }

    inline void FrAdcDataNonPtr::
    SetBias( bias_type Bias )
    {
      bias = Bias;
    }

    inline void FrAdcDataNonPtr::
    SetNBits( nBits_type NBits )
    {
      nBits = NBits;
    }

    inline void FrAdcDataNonPtr::
    SetSampleRate( sampleRate_type SampleRate )
    {
      sampleRate = SampleRate;
#if WORKING
      //-----------------------------------------------------------------
      /// \todo
      /// Modifications to the sample rate need also be reflected in the
      /// corrisponding FrVect structures.
      //-----------------------------------------------------------------
      for ( data_type::iterator
	      cur = RefData( ).begin( ),
	      last = RefData( ).end( );
	    cur != last;
	    ++cur )
      {
	cur->
      }
#endif /* WORKING */
    }

    inline void FrAdcDataNonPtr::
    SetSlope( slope_type Slope )
    {
      slope = Slope;
    }

    inline void FrAdcDataNonPtr::
    SetTimeOffset( timeOffset_type TimeOffset )
    {
      timeOffset = TimeOffset;
    }


    inline void FrAdcDataNonPtr::
    AppendComment( const comment_type& Comment )
    {
      FrameCPP::Common::AppendComment( comment, Comment );
    } 

    inline void FrAdcDataNonPtr::
    SetChannelGroup( channelGroup_type Value)
    {
      channelGroup = Value;
    }

    inline void FrAdcDataNonPtr::
    SetChannelNumber( channelNumber_type Value)
    {
      channelNumber = Value;
    }

    inline void FrAdcDataNonPtr::
    SetComment( const comment_type& Comment )
    {
      comment = Comment;
    } 

    inline bool FrAdcDataNonPtr::
    operator==( const FrAdcDataNonPtr& RHS ) const
    {
#if LM_INFO
#define LM_INFO_CHECK(a)						\
      if ( ! ( a == RHS.a ) )						\
      {									\
	std::cerr << "DEBUG: Comparison of " << #a << " failed" << std::endl; \
      }

      LM_INFO_CHECK( name );
      LM_INFO_CHECK( comment);
      LM_INFO_CHECK( channelGroup );
      LM_INFO_CHECK( channelNumber );
      LM_INFO_CHECK( nBits );
      LM_INFO_CHECK( bias );
      LM_INFO_CHECK( slope );
      LM_INFO_CHECK( units );
      LM_INFO_CHECK( fShift );
      LM_INFO_CHECK( timeOffset );
      LM_INFO_CHECK( dataValid );
      LM_INFO_CHECK( phase );
      LM_INFO_CHECK( data );
      LM_INFO_CHECK( aux );
#undef LM_INFO_CHECK
#endif /* LM_INFO */

      return ( ( this == &RHS ) ||
	       	  ( ( name == RHS.name )
		    && ( comment == RHS.comment )
		    && ( channelGroup == RHS.channelGroup )
		    && ( channelNumber == RHS.channelNumber )
		    && ( nBits == RHS.nBits )
		    && ( sampleRate == RHS.sampleRate )
		    && ( bias == RHS.bias )
		    && ( slope == RHS.slope )
		    && ( units == RHS.units )
		    && ( fShift == RHS.fShift )
		    && ( timeOffset == RHS.timeOffset )
		    && ( dataValid == RHS.dataValid )
		    && ( phase == RHS.phase )
		    ) );

    }

    typedef FR_ADC_DATA_PS_NAMESPACE :: FrAdcDataPST< FrVect, FrVect > FrAdcDataPS;

    //===================================================================
    /// \brief ADC Data
    //===================================================================
    class FrAdcData
      : public FrAdcDataNonPtr,
        public FrAdcDataPS,
	public ObjectWithChecksum< Common::FrameSpec::Object >,
	public Common::TOCInfo
    {
    public:
      typedef General::SharedPtr< FrAdcData >	subset_ret_type;
      typedef FrAdcDataPS			references_type;

      inline static const char* DEFAULT_UNITS( )
      {
	return FR_ADC_DATA_DEFAULT_UNITS( );
      }

      using FrAdcDataNonPtr::GetChannelGroup;

      //-----------------------------------------------------------------
      /// \brief Default constructor
      ///
      /// \return
      ///    A new instance of this object.
      //-----------------------------------------------------------------
      FrAdcData( );

      //-----------------------------------------------------------------
      /// \brief Copy Constructor
      ///
      /// \param[in] Source
      ///     The object from which to copy the information.
      ///
      /// \return
      ///    A new instance of this object.
      //-----------------------------------------------------------------
      FrAdcData( const FrAdcData& Source );

      //-----------------------------------------------------------------
      /// \brief Constructor
      ///
      /// \param[in] Name
      ///     The Channel name -- must be unique within the frame
      /// \param[in] Group
      ///     Channel grouping number containing ADC.
      /// \param[in] Channel
      ///     Channel number.
      /// \param[in] NBits
      ///     Number of bits in A/D output.
      /// \param[in] Bias
      ///     DC bias on channel (Units ADC counts = 0)
      /// \param[in] Slope
      ///     ADC calibration: input units/count.
      /// \param[in] Units
      ///     ADC calibration: input units for slope.
      /// \param[in] SampleRate
      ///     Data acquistion rate, samples/s.
      /// \param[in] FShift
      ///     The frequency (in Hz) in the origional data that
      ///     oorresponds to 0 Hz in the heterodyned series.
      /// \param[in] TimeOffset
      ///     Offset of the first sample relative to the frame start
      ///     time (seconds). Must be positive and smaller than the
      ///     frame length.
      /// \param[in] DataValid
      ///     Data valid flag.
      /// \param[in] Phase
      ///     Phase of heterodyning signal at start of dataset.
      ///
      /// \return
      ///     A new instance of the object.
      //-----------------------------------------------------------------
      FrAdcData( const name_type& Name,
		 channelGroup_type Group,
		 channelNumber_type Channel,
		 nBits_type NBits,
		 sampleRate_type SampleRate,
		 bias_type Bias = DEFAULT_BIAS,
		 slope_type Slope = DEFAULT_SLOPE,
		 const units_type& Units = DEFAULT_UNITS( ),
		 fShift_type FShift = DEFAULT_FSHIFT,
		 timeOffset_type TimeOffset = DEFAULT_TIME_OFFSET,
		 dataValid_type DataValid = DEFAULT_DATA_VALID,
		 phase_type Phase = DEFAULT_PHASE );

      //-----------------------------------------------------------------
      /// \brief Promotion Constructor
      ///
      /// \param[in] Source
      ///     An FrAdcData structure from a previous frame specification.
      /// \param[in] Stream
      ///     The stream from which the earliest version of the
      ///     FrAdcData structure was read.
      ///
      /// \return
      ///     A new instance of the object.
      //-----------------------------------------------------------------
      FrAdcData( Previous::FrAdcData& Source, istream_type* Stream );

      //-----------------------------------------------------------------
      /// \brief Destructor
      //-----------------------------------------------------------------
      ~FrAdcData( );

      //-----------------------------------------------------------------
      /// \brief Retrieve the name of the FrAdcData struture.
      //-----------------------------------------------------------------
      const std::string& GetNameSlow( ) const;

      //-----------------------------------------------------------------
      /// \brief The name structure name of this object.
      ///
      /// \return
      ///     The name of the structure as specified by the frame
      ///     specification.
      //-----------------------------------------------------------------
      static const char* StructName( );

      //-----------------------------------------------------------------
      /// \brief The description of structure
      ///
      /// \return
      ///     A Description object which describes this structure as
      ///     specified by the frame specification.
      //-----------------------------------------------------------------
      static const Common::Description* StructDescription( );

      //-----------------------------------------------------------------
      /// \brief equality operator
      ///
      /// \param[in] RHS
      ///     The FrAdcData object to be compared.
      ///
      /// \return
      ///     The value true is returned if this object is equivelent
      ///     to the RHS instance; false otherwise.
      //-----------------------------------------------------------------
      bool operator==( const FrAdcData& RHS ) const;

      //-----------------------------------------------------------------
      /// \brief inequality operator
      ///
      /// \param[in] RHS
      ///     The FrAdcData object to be compared.
      ///
      /// \return
      ///     The value true is returned if this object is not equivelent
      ///     to the RHS instance; false otherwise.
      //-----------------------------------------------------------------
      bool operator!=( const FrAdcData& RHS ) const;

      //-----------------------------------------------------------------
      /// \brief equality operator for abstract data type
      ///
      /// \param[in] Obj
      ///     The object to be compared.
      ///
      /// \return
      ///     The value true is returned if this object is equivelent
      ///     to the RHS instance; false otherwise.
      //-----------------------------------------------------------------
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

      //-----------------------------------------------------------------
      /// \brief The name structure name of this object.
      ///
      /// \return
      ///     The name of the structure as specified by the frame
      ///     specification.
      //-----------------------------------------------------------------
      virtual const char* ObjectStructName( ) const;

      //-----------------------------------------------------------------
      /// \brief Merge with another FrAdcData
      ///
      /// \param[in] RHS
      ///     The source of the information to append to this FrAdcData
      ///     structure.
      ///
      /// \return
      ///     A reference to this object
      //-----------------------------------------------------------------
      FrAdcData& Merge( const FrAdcData& RHS );

      //-----------------------------------------------------------------
      /// \brief Promote previous version of object to this version
      ///
      /// \param[in] Source
      ///     The frame specification version of the object.
      /// \param[in] Obj
      ///     The previous version of the object.
      /// \param[in] Stream
      ///     Stream from which the origional object was created.
      ///
      /// \return
      ///     A new instance of FrameH based on Obj
      //-----------------------------------------------------------------
      static promote_ret_type
      Promote( INT_2U Source,
	       promote_arg_type Obj,
	       istream_type* Stream );

      //-----------------------------------------------------------------
      /// \brief Request a subset of the data.
      ///
      /// \param[in] Offset
      ///     The number of seconds from the start of the FrAdcData
      ///     object from which to start extracting data.
      /// \param[in] Dt
      ///     The number of seconds of data in the resulting FrAdcData
      ///     strucutre.
      ///
      /// \return
      ///     A FrAdcData structure which contains Dt seconds of data
      ///     starting at Offset of this FrAdcData structure.
      //-----------------------------------------------------------------
      subset_ret_type Subset( REAL_8 Offset, REAL_8 Dt ) const;
   
#if WORKING_VIRTUAL_TOCQUERY
      //-----------------------------------------------------------------
      /// \brief Answer requests for information as needed by TOC
      ///
      /// \param[in] InfoClass
      ///     Class of information being requested.
      //-----------------------------------------------------------------
      virtual void TOCQuery( int InfoClass, ... ) const;
#endif /*  WORKING_VIRTUAL_TOCQUERY */

    protected:
      //-----------------------------------------------------------------
      /// \brief Down grade an object
      ///
      /// \param[in] Target
      ///     The version of the frame specification to demote too.
      /// \param[in] Obj
      ///     The version of the object to demote.
      /// \param[in] Stream
      ///     The input stream from which the original object was read.
      ///
      /// \return
      ///     An object of the previous generation.
      //-----------------------------------------------------------------
      virtual demote_ret_type
      demote( INT_2U Target,
	      demote_arg_type Obj,
	      istream_type* Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Upgrade an object
      ///
      /// \param[in] Target
      ///     The version of the frame specification to promote too.
      /// \param[in] Obj
      ///     The object to be promoted.
      /// \param[in] Stream
      ///     The input stream from which the original object was read.
      ///
      /// \return
      ///     An object of this class.
      //-----------------------------------------------------------------
      virtual promote_ret_type
      promote( INT_2U Target,
	       promote_arg_type Obj,
	       istream_type* Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Number of bytes needed to write this structure
      ///
      /// \param[in] Stream
      ///     The stream from which to the object is being read or
      ///     written.
      ///
      /// \return
      ///     The number of bytes need to read or write this object.
      //-----------------------------------------------------------------
      cmn_streamsize_type pBytes( const Common::StreamBase& Stream) const;

      //-----------------------------------------------------------------
      /// \brief Virtual constructor
      ///
      /// \param[in] Stream
      ///     The input stream from where the object is being read.
      ///
      /// \return
      ///     The newly allocated object.
      //-----------------------------------------------------------------
      virtual FrAdcData* pCreate( istream_type& Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Write the structure to the stream
      ///
      /// \param[in] Stream
      ///     The output stream where the object is to be written.
      //-----------------------------------------------------------------
      virtual void pWrite( ostream_type& Stream ) const;

#if ! WORKING_VIRTUAL_TOCQUERY
      //-----------------------------------------------------------------
      /// \brief Answer requests for information as needed by TOC
      ///
      /// \param[in] InfoClass
      ///     Class of information being requested.
      /// \param[in] vl
      ///     Variable argument list of requested data.
      //-----------------------------------------------------------------
      virtual void vTOCQuery( int InfoClass, va_list vl ) const;
#endif /* ! WORKING_VIRTUAL_TOCQUERY */

    private:
      //-----------------------------------------------------------------
      /// \brief Object with checksum data.
      //-----------------------------------------------------------------
      typedef ObjectWithChecksum< Common::FrameSpec::Object > object_type;

      using Common::FrameSpec::Object::Create;

      //-----------------------------------------------------------------
      /// \brief Stream Constructor
      ///
      /// \param[in] Stream
      ///     The stream from which the object is being read.
      ///
      /// \return
      ///    A new instance of this object.
      //-----------------------------------------------------------------
      FrAdcData( istream_type& Stream );

      void copy_core( const FrAdcData& Source );
    };

    inline FrAdcData::promote_ret_type FrAdcData::
    Promote( INT_2U Source,
	     promote_arg_type Obj,
	     istream_type* Stream )
    {
      return Object::PromoteObject< Previous::FrAdcData, FrAdcData >
	( DATA_FORMAT_VERSION, Source, Obj, Stream );
    }

    inline const char* FrAdcData::
    StructName( )
    {
      static const char* name = "FrAdcData";
      return name;
    }

    inline bool FrAdcData::
    operator==( const FrAdcData& RHS ) const
    {
      return ( ( this == &RHS ) ||
	       ( FrAdcDataNonPtr::operator==( RHS )
		 && references_type::operator==( RHS ) )
	       )
	;
    }

    inline bool FrAdcData::
    operator!=( const FrAdcData& RHS ) const
    {
      return ! ( *this == RHS );
    }

    inline void FrAdcData::
    copy_core( const FrAdcData& Source )
    {
      name = Source.name;
      comment = Source.comment;
      channelGroup = Source.channelGroup;
      channelNumber = Source.channelNumber;
      nBits = Source.nBits;
      sampleRate = Source.sampleRate;
      bias = Source.bias;
      slope = Source.slope;
      units = Source.units;
      fShift = Source.fShift;
      timeOffset = Source.timeOffset;
      dataValid = Source.dataValid;
      phase = Source.phase;
      aux = Source.aux;
    }
  } // namespace - Version_8
} // namespace FrameCPP
#endif /* defined(__cplusplus) && !defined(SWIGIMPORTED) */
#endif /* FrameCPP_VERSION_8_FrAdcData_HH */
